@@ -1,5 +1,5 @@
 /* $File: //member/autrijus/PAR/myldr/main.c $ $Author: autrijus $
-   $Revision: #2 $ $Change: 2003 $ $DateTime: 2002/11/05 19:36:54 $ */
+   $Revision: #3 $ $Change: 2026 $ $DateTime: 2002/11/06 23:01:32 $ */
 
 #define PERL_NO_GET_CONTEXT
 #include <EXTERN.h>
@@ -15,6 +15,42 @@
 
 static PerlInterpreter *my_perl;
 
+#ifdef HAS_PROCSELFEXE
+/* This is a function so that we don't hold on to MAXPATHLEN
+   bytes of stack longer than necessary
+ */
+STATIC void
+S_procself_val(pTHX_ SV *sv, char *arg0)
+{
+    char buf[MAXPATHLEN];
+    int len = readlink(PROCSELFEXE_PATH, buf, sizeof(buf) - 1);
+
+    /* On Playstation2 Linux V1.0 (kernel 2.2.1) readlink(/proc/self/exe)
+       includes a spurious NUL which will cause $^X to fail in system
+       or backticks (this will prevent extensions from being built and
+       many tests from working). readlink is not meant to add a NUL.
+       Normal readlink works fine.
+     */
+    if (len > 0 && buf[len-1] == '\0') {
+      len--;
+    }
+
+    /* FreeBSD's implementation is acknowledged to be imperfect, sometimes
+       returning the text "unknown" from the readlink rather than the path
+       to the executable (or returning an error from the readlink).  Any valid
+       path has a '/' in it somewhere, so use that to validate the result.
+       See http://www.freebsd.org/cgi/query-pr.cgi?pr=35703
+    */
+    if (len > 0 && memchr(buf, '/', len)) {
+        sv_setpvn(sv,buf,len);
+    }
+    else {
+        sv_setpv(sv,arg0);
+    }
+}
+#endif /* HAS_PROCSELFEXE */
+
+
 char** prepare_args( int argc, char** argv, int* my_argc )
 {
     int i, count = ( argc ? argc : 1 ) + 3;
@@ -41,7 +77,6 @@ int main( int argc, char **argv, char **env )
     int i;
     char **fakeargv;
     GV* tmpgv;
-    SV* tmpsv;
     int options_count;
 
     if (!PL_do_undump) {
@@ -87,19 +122,25 @@ int main( int argc, char **argv, char **env )
     TAINT;
 
     if ((tmpgv = gv_fetchpv("0", TRUE, SVt_PV))) {/* $0 */
-	tmpsv = GvSV(tmpgv);
-	sv_setpv(tmpsv, argv[0]);
-	SvSETMAGIC(tmpsv);
+#ifdef HAS_PROCSELFEXE
+        S_procself_val(aTHX_ GvSV(tmpgv), argv[0]);
+#else
+#ifdef OS2
+        sv_setpv(GvSV(tmpgv), os2_execname(aTHX));
+#else
+        sv_setpv(GvSV(tmpgv), argv[0]);
+#endif
+#endif
+        SvSETMAGIC(GvSV(tmpgv));
     }
 
     if ((tmpgv = gv_fetchpv("\030",TRUE, SVt_PV))) {/* $^X */
-        tmpsv = GvSV(tmpgv);
 #ifdef WIN32
-        sv_setpv(tmpsv,"perl.exe");
+        sv_setpv(GvSV(tmpgv),"perl.exe");
 #else
-        sv_setpv(tmpsv,"perl");
+        sv_setpv(GvSV(tmpgv),"perl");
 #endif
-        SvSETMAGIC(tmpsv);
+        SvSETMAGIC(GvSV(tmpgv));
     }
 
     TAINT_NOT;
